package at.xaxa.gamelibraryapp.ui.Search

import android.util.Log
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import at.xaxa.gamelibraryapp.data.Game
import at.xaxa.gamelibraryapp.data.GameRepository
import at.xaxa.gamelibraryapp.data.db.LibraryEntity
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext


// UI State to hold a list of games
data class GameListUi(
    val games: Flow<List<Game>> = flow{emit(emptyList()) }
)

class SearchListViewModel(
    private val savedStateHandle: SavedStateHandle,
    private val gameRepository: GameRepository
) : ViewModel() {

    var editUiState by mutableStateOf(GameListUi())
        private set

    // Function to search games based on search text
    fun searchGames(searchText: String) {
        viewModelScope.launch {
            // Launching a background task to fetch the data from the repository
            val games = withContext(Dispatchers.IO) {
                gameRepository.searchGames(searchText) // Get the list of games
            }

            // Now transform the data before emitting
            val mappedGames = games.map { game ->
                // Transform each game as needed
                Game(
                    id = game.id,
                    name = game.name,
                    details = game.genres.firstOrNull()?.name ?: "Genre Unknown",
                    note = "",
                    imageUrl = game.background_image,
                    inLibrary = false
                )
            }

            editUiState = GameListUi(games = flow { emit(mappedGames) })
        }
    }

    fun addGameToDB(game: Game){
        viewModelScope.launch {
            try {
                val libraryEntity = LibraryEntity(
                    _id = game.id,
                    name = game.name,
                    details = game.details,
                    note = game.note,
                    imageUrl = game.imageUrl,
                    inLibrary = false
                )

                withContext(Dispatchers.IO) {
                    gameRepository.addEntry(libraryEntity) // Add game to the database
                }
                // Optionally, log or update UI state to reflect that the game was added
                Log.d("GameLibraryApp", "Game added to DB: ${game.name}")
            } catch (e: Exception) {
                Log.e("GameLibraryApp", "Error adding game to DB", e)
            }
        }
    }
}
